@@ -26,6 +26,7 @@
  *
  * History:
  *   0.1  26.06.96  Started
+ *   0.2  11.05.97  introduced hdrvcomm.h
  */
 
 /*****************************************************************************/
@@ -35,38 +36,26 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
 #include <unistd.h>
 #include <string.h>
-#include <sys/ioctl.h>
-#include <signal.h>
-#include <sys/types.h>
-#include <sys/socket.h>
 #include <linux/if.h>
-#include <netinet/in.h>
-#include <linux/if_ether.h>
-#include <linux/hdlcdrv.h>
-#include <linux/soundmodem.h>
+#include "hdrvcomm.h"
 
 /* ---------------------------------------------------------------------- */
 
-static struct ifreq ifr_sm;
 static char *progname;
-static int sock_sm;
 static unsigned int mixdevice;
 
 /* ---------------------------------------------------------------------- */
 
 static int do_mix_ioctl(int cmd, struct sm_mixer_data *mixdat)
 {
-	struct ifreq ifr = ifr_sm;
 	struct sm_ioctl par;
 	int i;
 
-	ifr.ifr_data = (caddr_t)&par;
 	par.cmd = cmd;
 	par.data.mix = *mixdat;
-	i = ioctl(sock_sm, SIOCDEVPRIVATE, &ifr);
+	i = hdrvc_sm_ioctl(cmd, &par);
 	*mixdat = par.data.mix;
 	return i;
 }
@@ -730,31 +719,20 @@ static const char *usage_str =
 
 int main(int argc, char *argv[])
 {
-	char *name_sm = "sm0";
 	int c;
 	struct sm_mixer_data mixdat;
 
 	progname = *argv;
-	printf("%s: Version 0.1; (C) 1996 by Thomas Sailer HB9JNX/AE4WA\n", *argv);
-	while ((c = getopt(argc, argv, "i:")) != -1) {
+	printf("%s: Version 0.2; (C) 1996-1997 by Thomas Sailer HB9JNX/AE4WA\n", *argv);
+	hdrvc_args(&argc, argv, "sm0");
+	while ((c = getopt(argc, argv, "")) != -1) {
 		switch (c) {
-		case 'i':
-			name_sm = optarg;
-			break;
 		default:
 			printf("usage: %s %s", *argv, usage_str);
 			exit(-1);
 		}
 	}
-	if ((sock_sm = socket(PF_INET, SOCK_PACKET, htons(ETH_P_AX25))) < 0) {
-		perror("socket");
-		exit(-1);
-	}
-	strcpy(ifr_sm.ifr_name, name_sm);
-	if (ioctl(sock_sm, SIOCGIFFLAGS, &ifr_sm) < 0) {
-		perror("ioctl: SIOCGIFFLAGS");
-		exit(-1);
-	}
+	hdrvc_init();
 	mixdat.reg = 0;
 	if (do_mix_ioctl(SMCTL_GETMIXER, &mixdat) < 0) {
 		perror("do_mix_ioctl: SMCTL_GETMIXER");
