@@ -45,7 +45,7 @@ static void invert_ssid(char *out, char *in)
  * Initiate a AX.25, NET/ROM, ROSE or TCP connection to the host
  * specified by `address'.
  */
-static int connect_to(char *address[], int family)
+static int connect_to(char *address[], int family, int escape)
 {
 	int fd;
 	fd_set read_fdset;
@@ -88,14 +88,23 @@ static int connect_to(char *address[], int family)
 			close(fd);
 			return -1;
 		}
-		sprintf(User.dl_name, "%s @ %s", address[0], address[1]);
+		memset(path, 0, 11);
+		memcpy(path, rs_config_get_addr(NULL), 4);
+		addrlen = strlen(address[1]);
+		if ((addrlen != 6) && (addrlen != 10))
+		{
+			node_msg("Invalid ROSE address");
+			return(-1);
+		}
+		memcpy(path + (10-addrlen), address[1], addrlen);
+		sprintf(User.dl_name, "%s @ %s", address[0], path);
 		sockaddr.rose.srose_family = AF_ROSE;
 		sockaddr.rose.srose_ndigis = 0;
 		if (convert_call_entry(address[0], sockaddr.rose.srose_call.ax25_call) == -1) {
 			close(fd);
 			return -1;
 		}
-		if (convert_rose_address(address[1], sockaddr.rose.srose_addr.rose_addr) == -1) {
+		if (convert_rose_address(path, sockaddr.rose.srose_addr.rose_addr) == -1) {
 			close(fd);
 			return -1;
 		}
@@ -131,8 +140,15 @@ static int connect_to(char *address[], int family)
 			return -1;
 		}
 		if ((np = find_node(address[0], NULL)) == NULL) {
+#if 1
 			node_msg("No such node");
 			return -1;
+#else
+			np = calloc(1, sizeof(struct proc_nr_nodes));
+			strupr(address[0]);
+			strcpy(np->call, address[0]);
+			strcpy(np->alias, "*");
+#endif
 		}
 		strcpy(User.dl_name, print_node(np->alias, np->call));
 		if (convert_call(np->call, &sockaddr.ax25) == -1) {
@@ -304,7 +320,13 @@ static int connect_to(char *address[], int family)
 			}
 		}
 	}
-	node_msg("Connected to %s", User.dl_name);
+	if (escape == -1)
+		node_msg("Connected to %s", User.dl_name);
+	else
+		node_msg("Connected to %s (Escape: %s%c)",
+			 User.dl_name,
+			 escape < 32 ? "CTRL-" : "",
+			 escape < 32 ? (escape + 'A' - 1) : escape);
 	usflush(User.fd);
 	log(LOGLVL_GW, "Connected to %s", User.dl_name);
 	if (init_io(fd, paclen, eol) == -1) {
@@ -333,8 +355,9 @@ static int connect_to(char *address[], int family)
 
 int do_connect(int argc, char **argv)
 {
-	int fd, c, family, stay;
+	int fd, c, family, stay, escape;
 	fd_set fdset;
+	char *connstr = NULL;
 
 	stay = ReConnectTo;
 	if (!strcasecmp(argv[argc - 1], "s")) {
@@ -355,16 +378,23 @@ int do_connect(int argc, char **argv)
 		family = AF_INET;
 	else if (argc == 2)
 		family = AF_NETROM;
-	else if (strlen(argv[2]) == 10)
+	else if (strspn(argv[2], "0123456789") == strlen(argv[2]))
 		family = AF_ROSE;
 	else
 		family = AF_AX25;
-	if ((fd = connect_to(++argv, family)) == -1) {
+	if (family == AF_INET && argc > 3)
+		connstr = argv[3];
+	escape = (check_perms(PERM_NOESC, 0L) == 0) ? -1 : EscChar;
+	if ((fd = connect_to(++argv, family, escape)) == -1) {
 		set_eolmode(User.fd, EOLMODE_TEXT);
 		if (fcntl(User.fd, F_SETFL, 0) == -1)
 			node_perror("do_connect: fcntl - stdin", errno);
 		return 0;
 	}
+	if (connstr) {
+		usprintf(fd, "%s\n", connstr);
+		usflush(fd);
+	}
 	while (1) {
 		FD_ZERO(&fdset);
 		FD_SET(fd, &fdset);
@@ -385,8 +415,15 @@ int do_connect(int argc, char **argv)
 		}
 		if (FD_ISSET(User.fd, &fdset)) {
 			alarm(ConnTimeout);
-			while((c = usgetc(User.fd)) != -1)
+			while((c = usgetc(User.fd)) != -1) {
+				if (escape != -1 && c == escape)
+					break;
 				usputc(c, fd);
+			}
+			if (escape != -1 && c == escape) {
+				readline(User.fd);
+				break;
+			}
 			if (errno != EAGAIN) {
 				stay = 0;
 				break;
@@ -404,39 +441,29 @@ int do_connect(int argc, char **argv)
 			node_perror("do_connect: fcntl - stdin", errno);
 		node_msg("Reconnected to %s", NodeId);
 	} else
-		do_bye(0, NULL);
+		logout("No reconnect");
 	return 0;
 }
 
 int do_finger(int argc, char **argv)
 {
 	int fd, c;
-	char *name, *addr[3], *defaulthost, *cp;
+	char *name, *addr[3], *cp;
 
-	if (*argv[0] == 'c') {
-		if (argc < 2) {
-			node_msg("Usage: callbook <call>[@<server>]");
-			return 0;
-		}
-		defaulthost = CallServer;
-		addr[1] = CallServerPort;
-	} else {
-		defaulthost = "localhost";
-		addr[1] = "finger";
-	}
 	if (argc < 2) {
 		name = "";
-		addr[0] = defaulthost;
+		addr[0] = "localhost";
 	} else if ((cp = strchr(argv[1], '@')) != NULL) {
 		*cp = 0;
 		name = argv[1];
 		addr[0] = ++cp;
 	} else {
 		name = argv[1];
-		addr[0] = defaulthost;
+		addr[0] = "localhost";
 	}
+	addr[1] = "finger";
 	addr[2] = NULL;
-	if ((fd = connect_to(addr, AF_INET)) != -1) {
+	if ((fd = connect_to(addr, AF_INET, -1)) != -1) {
 		if (fcntl(fd, F_SETFL, 0) == -1)
 			node_perror("do_finger: fcntl - fd", errno);
 		init_io(fd, 1024, INET_EOL);
